/**
 *
 * Copyright 2000, 2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 *
**/
#include <stdio.h>
#include <string.h>
#include "loadbalancer.h"

const char* HTTPS = "https://";
const char* HTTP = "http://";
//pointer to beginning of Http listeners
struct http_listener *http_listeners;
//pointer to beginning of Https listeners
struct http_listener *https_listeners;
//current position of Http listener
int currentHttpPos;
//current position of Https listener
int currentHttpsPos;
//number of active Http listeners
int activeHttpListSize;
//number of active Https listeners
int activeHttpsListSize;

/**
 *
 * Function Name: lb_policy_init 
 *
 * Description : Initialization method for Userdefined load balancer module. Method called whenever there is a change in the active listener list.
 *
 */

int lb_policy_init(struct http_listener listener[], int noOfListeners){
    int x = 0;    
    printf("There are %d listeners\n", noOfListeners);
    //The list is parsed based on assumption that listeners is provided in below format
    //first all Http listeners
    //then all Https listeners
    //So two pointer and counts are maintained. 
    //One for all Http listeners and other for all Https listeners
    http_listeners = listener; 
    https_listeners = NULL; 
    activeHttpsListSize = 0;
    for(x = 0; x < noOfListeners; x++){
        printf("URL=%s weight=%d \n", listener[x].url, listener[x].weight);
    }
    for(x = 0; x < noOfListeners; x++){
        if(strstr(listener[x].url, HTTPS) ){
	    https_listeners = &listener[x];
	    activeHttpListSize = x;
	    activeHttpsListSize = noOfListeners - activeHttpListSize;
	    break;
	}
    }
    currentHttpPos = -1;
    currentHttpsPos = -1;
    return 0;
}

/**
 *
 * Function Name: lb_decision 
 *
 * Description : Decision method for choosing a listener for an incoming request. Method is called when a active listener is to be choosen for an incoming request. 
 *
 */
char * lb_decision(int secure, char *url, struct header headers[], int noOfHeaders){
    int x;
    int forceSecure = secure && strstr(url, HTTPS);
    
    printf("Headers:\n");
    for(x = 0; x < noOfHeaders; x++){
        printf("%s=%s\n", headers[x].name,headers[x].value);
   }
   if(forceSecure && activeHttpsListSize > 0){
        currentHttpsPos = (currentHttpsPos + 1) % activeHttpsListSize ; 
        printf("Returning the listener: %s\n", https_listeners[currentHttpsPos].url);
        return https_listeners[currentHttpsPos].url;
    }else if(!forceSecure && activeHttpListSize > 0){
        currentHttpPos = (currentHttpPos + 1) % activeHttpListSize ; 
        printf("Returning the listener: %s\n", http_listeners[currentHttpPos].url);
        return http_listeners[currentHttpPos].url;
    }        
    printf("Unable to find a matching daemon for the request %s\n", url);
    return NULL; 
}
